USE AdventureWorks2014
GO

--find table with the most rows
SELECT 
	TableSchema = S.name,
	TableName = T.name,
	FullName = QUOTENAME(S.name) + '.' + QUOTENAME(T.name),
	RecordCount = SUM(P.[rows])
FROM sys.tables T
JOIN sys.schemas S ON T.schema_id = S.schema_id
JOIN sys.indexes I ON T.object_id = I.object_id
JOIN sys.partitions P ON I.object_id = P.object_id AND I.index_id = P.index_id
WHERE T.is_ms_shipped = 0
	AND I.[type] < 2 --heap (0); clustered (1)
	--AND p.rows > 0
GROUP BY
	T.name, S.name
ORDER BY RecordCount DESC;

--check out what indexes it has on it we can use
EXECUTE sp_help 'SALES.SalesOrderDetail';
--rowguid
--product id
--salesorderid, salesorderdetailid (clustered)

--lets see what statistics we have	
EXECUTE sp_helpstats 'Sales.SalesOrderDetail', 'ALL';

--more detailed information
SELECT  [sch].[name] + '.' + [so].[name] AS [TableName] ,
    [si].[index_id] AS [Index ID] ,
    [ss].[name] AS [Statistic] ,
    STUFF(( SELECT  ', ' + [c].[name]
            FROM    [sys].[stats_columns] [sc]
                    JOIN [sys].[columns] [c]
                     ON [c].[column_id] = [sc].[column_id]
                        AND [c].[object_id] = [sc].[OBJECT_ID]
            WHERE   [sc].[object_id] = [ss].[object_id]
                    AND [sc].[stats_id] = [ss].[stats_id]
            ORDER BY [sc].[stats_column_id]
          FOR
            XML PATH('')
          ), 1, 2, '') AS [ColumnsInStatistic] ,
    [ss].[auto_Created] AS [WasAutoCreated] ,
    [ss].[user_created] AS [WasUserCreated] ,
    [ss].[has_filter] AS [IsFiltered] ,
    [ss].[filter_definition] AS [FilterDefinition] ,
    [ss].[is_temporary] AS [IsTemporary]
FROM    [sys].[stats] [ss]
    JOIN [sys].[objects] AS [so] ON [ss].[object_id] = [so].[object_id]
    JOIN [sys].[schemas] AS [sch] ON [so].[schema_id] = [sch].[schema_id]
    LEFT OUTER JOIN [sys].[indexes] AS [si]
          ON [so].[object_id] = [si].[object_id]
             AND [ss].[name] = [si].[name]
WHERE   [so].[object_id] = OBJECT_ID(N'Sales.SalesOrderDetail')
ORDER BY [ss].[user_created] ,
    [ss].[auto_created] ,
    [ss].[has_filter];

--lets check out product id distribution
DBCC SHOW_STATISTICS ("SALES.SalesOrderDetail",IX_SalesOrderDetail_ProductID);

/*
as Erin Stellato stated in her blog on statistics (--https://www.simple-talk.com/sql/performance/managing-sql-server-statistics/)
"The Updated column in the statistics header displays the last time statistic was updated, whether it was via an automatic or manual update. We can use the Rows and Rows_Sampled columns to determine whether the sample rate was 100% (FULLSCAN) or something less.
The density vector, which is the second set of information, provides detail regarding the uniqueness of the columns. The optimizer can use the density value for the left-based column in the key to estimate how many rows would return for any given value in the table. Density is calculated as "1/unique number of values in the column".
The final section of the output, the histogram, provides detailed information, also used by the optimizer. A histogram can have a maximum of 200 steps, and the histogram displays the number of rows equal to each step, as well as number of rows between steps, number of distinct values in the step, and the average number of rows per value in the step.
"

Looking at our density vector we have 0.003759399 or 265.999964 distinct values which we can check by running
SELECT DISTINCT ProductId FROM Sales.SalesOrderDetail --266 distinct ProductIds

Looking at our histogram we can see the RANGE_HI_KEY matches our ProductIds and EQ_ROWS matches the number of records corresponding to that ID
SELECT ProductId, COUNT(*) as RecordCount FROM Sales.SalesOrderDetail GROUP BY ProductId ORDER BY 1 ASC
*/

--returning just product id we get a seek on the index - IX_SalesOrderDetail_ProductID
SELECT 
	SOD.ProductId
FROM SALES.SalesOrderDetail SOD
WHERE SOD.ProductID < 715;

--because we have so few records in the table 121317 it is not worth the optimizer doing a bookmark lookup here. 
--It just scans the clustered index (PK_SalesOrderDetail_SalesOrderID_SalesOrderDetailID) where it has all of the data.
SELECT 
	SOD.ProductId, SOD.OrderQty 
FROM SALES.SalesOrderDetail SOD
WHERE SOD.ProductID < 715;

--productID 744 has 13 records lets see if we get a seek and a lookup... we do
SELECT 
	SOD.ProductId, SOD.OrderQty 
FROM SALES.SalesOrderDetail SOD
WHERE SOD.ProductID = 744;

--ok that was a lot of setup and previously it was very data dependent. Lets test out this new command DBCC Clone Database.
--12.0.4459
DBCC CLONEDATABASE (AdventureWorks2014, AdventureWorks2014_Clone);

GO

--now on the clone database 
USE AdventureWorks2014_Clone
GO

SELECT 
	TableSchema = S.name,
	TableName = T.name,
	FullName = QUOTENAME(S.name) + '.' + QUOTENAME(T.name),
	RecordCount = SUM(P.[rows])
FROM sys.tables T
JOIN sys.schemas S ON T.schema_id = S.schema_id
JOIN sys.indexes I ON T.object_id = I.object_id
JOIN sys.partitions P ON I.object_id = P.object_id AND I.index_id = P.index_id
WHERE T.is_ms_shipped = 0
	AND I.[type] < 2 --heap (0); clustered (1)
	--AND p.rows > 0
GROUP BY
	T.name, S.name
ORDER BY RecordCount DESC;


--lets check out our execution plans for the queries
--returning just product id we get a seek on the index - IX_SalesOrderDetail_ProductID
SELECT 
	SOD.ProductId
FROM SALES.SalesOrderDetail SOD
WHERE SOD.ProductID < 715;

--because we have so few records in the table 121317 it is not worth the optimizer doing a bookmark lookup here. 
--It just scans the clustered index (PK_SalesOrderDetail_SalesOrderID_SalesOrderDetailID) where it has all of the data.
SELECT 
	SOD.ProductId, SOD.OrderQty 
FROM SALES.SalesOrderDetail SOD
WHERE SOD.ProductID < 715;

--productID 744 has 13 records lets see if we get a seek and a lookup... we do
SELECT 
	SOD.ProductId, SOD.OrderQty 
FROM SALES.SalesOrderDetail SOD
WHERE SOD.ProductID = 744;


--it is in read only mode. let's try to make it read / write able
ALTER DATABASE AdventureWorks2014_Clone SET READ_WRITE WITH NO_WAIT



--now what I wanted to test. if we create a covering index (remember no data in this database)
--what happens to my execution plan. 
CREATE NONCLUSTERED INDEX IX_SALES_SalesOrderDetail_CoverTest1
ON [Sales].[SalesOrderDetail] ([ProductID]) INCLUDE ([OrderQty]);


--now selecting this query will hopefully give me a plan with a seek using IX_SALES_SalesOrderDetail_CoverTest1
SELECT 
	SOD.ProductId, SOD.OrderQty 
FROM SALES.SalesOrderDetail SOD
WHERE SOD.ProductID < 715;


DROP INDEX IX_SALES_SalesOrderDetail_CoverTest1
ON [Sales].[SalesOrderDetail] 
